@@ -2,20 +2,20 @@
  *
  * Copyright (C) 2005 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -108,7 +108,7 @@ gda_data_model_iter_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelIterClass),
 			(GBaseInitFunc) NULL,
@@ -285,28 +285,26 @@ param_changed_cb (GdaParameterList *paramlist, GdaParameter *param)
 	gint col;
 
 	iter = GDA_DATA_MODEL_ITER (paramlist);
-	if (iter->priv->keep_param_changes ||
-	    (iter->priv->row < 0))
-		return;
-
-	g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [0]);
-	g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [1]);
-
-	/* propagate the value update to the data model */
-	col = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "model_col")) - 1;
-	g_return_if_fail (col >= 0);
-
-	if (! gda_data_model_set_value_at (GDA_DATA_MODEL (iter->priv->data_model), 
-					   col, iter->priv->row, gda_parameter_get_value (param), NULL)) {
-		/* writing to the model failed, revert back the change to parameter */
-		iter->priv->keep_param_changes = TRUE;
-		gda_parameter_set_value (param, gda_data_model_get_value_at (GDA_DATA_MODEL (iter->priv->data_model), 
-									     col, iter->priv->row)); 
-		iter->priv->keep_param_changes = FALSE;
+	if (!iter->priv->keep_param_changes && (iter->priv->row >= 0)) {
+		g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [0]);
+		g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [1]);
+		
+		/* propagate the value update to the data model */
+		col = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "model_col")) - 1;
+		g_return_if_fail (col >= 0);
+		
+		if (! gda_data_model_set_value_at (GDA_DATA_MODEL (iter->priv->data_model), 
+						   col, iter->priv->row, gda_parameter_get_value (param), NULL)) {
+			/* writing to the model failed, revert back the change to parameter */
+			iter->priv->keep_param_changes = TRUE;
+			gda_parameter_set_value (param, gda_data_model_get_value_at (GDA_DATA_MODEL (iter->priv->data_model), 
+										     col, iter->priv->row)); 
+			iter->priv->keep_param_changes = FALSE;
+		}
+		
+		g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [0]);
+		g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [1]);
 	}
-	
-	g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [0]);
-	g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [1]);
 
 	/* for the parent class */
 	if (((GdaParameterListClass *) parent_class)->param_changed)
@@ -326,28 +324,26 @@ param_attr_changed_cb (GdaParameterList *paramlist, GdaParameter *param)
 	gboolean toset;
 
 	iter = GDA_DATA_MODEL_ITER (paramlist);
-	if (iter->priv->keep_param_changes ||
-	    (iter->priv->row < 0))
-		return;
-
 	if (!GDA_IS_DATA_PROXY (iter->priv->data_model))
 		return;
 
-	g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [0]);
-	g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [1]);
-
-	/* propagate the value update to the data model */
-	col = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "model_col")) - 1;
-	g_return_if_fail (col >= 0);
-
-	g_object_get (G_OBJECT (param), "use-default-value", &toset, NULL);
-	if (toset && gda_parameter_get_exists_default_value (param))
-		gda_data_proxy_alter_value_attributes (GDA_DATA_PROXY (iter->priv->data_model), 
-						       iter->priv->row, col, 
-						       GDA_VALUE_ATTR_CAN_BE_DEFAULT | GDA_VALUE_ATTR_IS_DEFAULT);
-	
-	g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [0]);
-	g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [1]);
+	if (!iter->priv->keep_param_changes && (iter->priv->row >= 0)) {
+		g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [0]);
+		g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [1]);
+		
+		/* propagate the value update to the data model */
+		col = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "model_col")) - 1;
+		g_return_if_fail (col >= 0);
+		
+		g_object_get (G_OBJECT (param), "use-default-value", &toset, NULL);
+		if (toset && gda_parameter_get_exists_default_value (param))
+			gda_data_proxy_alter_value_attributes (GDA_DATA_PROXY (iter->priv->data_model), 
+							       iter->priv->row, col, 
+							       GDA_VALUE_ATTR_CAN_BE_DEFAULT | GDA_VALUE_ATTR_IS_DEFAULT);
+		
+		g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [0]);
+		g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [1]);
+	}
 
 	/* for the parent class */
 	if (((GdaParameterListClass *) parent_class)->param_attr_changed)
@@ -459,9 +455,9 @@ gda_data_model_iter_set_property (GObject *object,
 					gda_parameter_set_exists_default_value (param, TRUE);
 				gda_parameter_list_add_param ((GdaParameterList *) iter, param);
 				g_object_set_data (G_OBJECT (param), "model_col", GINT_TO_POINTER (col + 1));
-				g_object_unref (param);
-				gda_object_connect_destroy (ptr,
+				gda_object_connect_destroy (param,
 							    G_CALLBACK (destroyed_param_cb), iter);
+				g_object_unref (param);
 			}
 		}
 		case PROP_FORCED_MODEL: {
@@ -715,6 +711,55 @@ gda_data_model_iter_get_param_for_column (GdaDataModelIter *iter, gint col)
 	return g_slist_nth_data (((GdaParameterList *) iter)->parameters, col);
 }
 
+/**
+ * gda_data_model_iter_get_value_at
+ * @iter: a #GdaDataModelIter object
+ * @col: the requested column
+ *
+ * Get the value stored at the column @col in @iter
+ *
+ * Returns: the #GValue, or %NULL
+ */
+const GValue *
+gda_data_model_iter_get_value_at (GdaDataModelIter *iter, gint col)
+{
+	GdaParameter *param;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), NULL);
+	g_return_val_if_fail (iter->priv, NULL);
+
+	param = (GdaParameter *) g_slist_nth_data (((GdaParameterList *) iter)->parameters, col);
+	if (param)
+		return gda_parameter_get_value (param);
+	else
+		return NULL;
+}
+
+/**
+ * gda_data_model_iter_get_value_for_field
+ * @iter: a #GdaDataModelIter object
+ * @field_name: the requested column name
+ *
+ * Get the value stored at the column @field_name in @iter
+ *
+ * Returns: the #GValue, or %NULL
+ */
+const GValue *
+gda_data_model_iter_get_value_for_field (GdaDataModelIter *iter, const gchar *field_name)
+{
+	GdaParameter *param;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), NULL);
+	g_return_val_if_fail (iter->priv, NULL);
+
+	param = gda_parameter_list_find_param ((GdaParameterList *) iter, field_name);
+	if (param)
+		return gda_parameter_get_value (param);
+	else
+		return NULL;
+}
+
+
 #ifdef GDA_DEBUG
 static void
 gda_data_model_iter_dump (GdaDataModelIter *iter, guint offset)
