@@ -2,20 +2,20 @@
  *
  * Copyright (C) 2003 - 2006 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -102,6 +102,7 @@ struct _GdaObjectRefPrivate
 	gchar                 *obj_name; /* real name of the object as returned by gda_object_get_name() */
 	gboolean               block_signals;
 	GdaObject             *helper_ref; /* a GdaQuery (for target resolution) or a GdaObjectRef (for a field resolution) */
+	gboolean               ref_losing;
 };
 
 
@@ -120,7 +121,7 @@ gda_object_ref_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaObjectRefClass),
 			(GBaseInitFunc) NULL,
@@ -200,6 +201,7 @@ gda_object_ref_init (GdaObjectRef * ref)
 	ref->priv->obj_name = NULL;
 	ref->priv->block_signals = FALSE;
 	ref->priv->helper_ref = NULL;
+	ref->priv->ref_losing = FALSE;
 }
 
 
@@ -314,6 +316,8 @@ destroyed_object_cb (GObject *obj, GdaObjectRef *ref)
 		g_object_unref (ref->priv->ref_object);
 
 	ref->priv->ref_object = NULL;
+	ref->priv->ref_losing = TRUE;
+	g_object_ref (ref);
 #ifdef GDA_DEBUG_signal
 	g_print (">> 'REF_LOST' from %s\n", __FUNCTION__);
 #endif
@@ -321,6 +325,9 @@ destroyed_object_cb (GObject *obj, GdaObjectRef *ref)
 #ifdef GDA_DEBUG_signal
 	g_print ("<< 'REF_LOST' from %s\n", __FUNCTION__);
 #endif
+	ref->priv->ref_object = NULL;
+	ref->priv->ref_losing = FALSE;
+	g_object_unref (ref);
 }
 
 static void
@@ -443,27 +450,50 @@ gda_object_ref_get_property (GObject *object,
 static GType
 handled_object_type (GType type)
 {
-	GType retval = 0;
+	if(type == 0)
+		return type;
 
-	/* types accepted AS IS */
-	if ((type == GDA_TYPE_DICT_TABLE) ||
-	    (type == GDA_TYPE_DICT_FIELD) ||
-	    (type == GDA_TYPE_QUERY) ||
-	    (type == GDA_TYPE_QUERY_TARGET) ||
-	    (type == GDA_TYPE_ENTITY_FIELD) ||
-	    (type == GDA_TYPE_DICT_FUNCTION) ||
-	    (type == GDA_TYPE_DICT_AGGREGATE) ||
-	    (type == GDA_TYPE_QUERY_FIELD))
-		retval = type;
+	/* Check whether the type is one of these types, 
+	 * or derived from these types.
+	 * TODO: Isn't there a glib macro for checking this without using the for loop?
+         */
+        GType type_to_check = type;
+	for(; type_to_check != G_TYPE_OBJECT; type_to_check = g_type_parent(type_to_check))
+	{
+		if(type_to_check == 0)
+			break;
+ 
+		/* type conversion */
+		if ((type_to_check == GDA_TYPE_QUERY_FIELD_ALL) ||
+		    (type_to_check == GDA_TYPE_QUERY_FIELD_FIELD) ||
+		    (type_to_check == GDA_TYPE_QUERY_FIELD_VALUE) ||
+		    (type_to_check == GDA_TYPE_QUERY_FIELD_FUNC))
+			return GDA_TYPE_QUERY_FIELD;
+	}
 
-	/* type conversion */
-	if ((type == GDA_TYPE_QUERY_FIELD_ALL) ||
-	    (type == GDA_TYPE_QUERY_FIELD_FIELD) ||
-	    (type == GDA_TYPE_QUERY_FIELD_VALUE) ||
-	    (type == GDA_TYPE_QUERY_FIELD_FUNC))
-		retval = GDA_TYPE_QUERY_FIELD;
+	/* Check whether the type is one of these types, 
+	 * or derived from these types.
+	 * TODO: Isn't there a glib macro for checking this without using the for loop?
+         */
+        type_to_check = type;
+	for(; type_to_check != G_TYPE_OBJECT; type_to_check = g_type_parent(type_to_check))
+	{
+		if(type_to_check == 0)
+			break;
 
-	return retval;
+		/* types accepted AS IS */
+		if ((type_to_check == GDA_TYPE_DICT_TABLE) ||
+		    (type_to_check == GDA_TYPE_DICT_FIELD) ||
+		    (type_to_check == GDA_TYPE_QUERY) ||
+		    (type_to_check == GDA_TYPE_QUERY_TARGET) ||
+		    (type_to_check == GDA_TYPE_ENTITY_FIELD) ||
+		    (type_to_check == GDA_TYPE_DICT_FUNCTION) ||
+		    (type_to_check == GDA_TYPE_DICT_AGGREGATE) ||
+		    (type_to_check == GDA_TYPE_QUERY_FIELD))
+			return type_to_check;
+	}
+
+	return 0;
 }
 
 /**
@@ -973,22 +1003,30 @@ gda_object_ref_activate (GdaObjectRef *ref)
 
 	/* Object treatment */
 	if (obj) {
-		if (ref->priv->increase_ref_object)
-			g_object_ref (obj);
-
-		gda_object_connect_destroy (obj, G_CALLBACK (destroyed_object_cb), ref);
-		ref->priv->ref_object = obj;
-
-		g_free (ref->priv->obj_name);
-		ref->priv->obj_name = g_strdup (gda_object_get_name (obj));
-
+		if (!ref->priv->ref_losing) {
+			if (ref->priv->increase_ref_object)
+				g_object_ref (obj);
+			
+			gda_object_connect_destroy (obj, G_CALLBACK (destroyed_object_cb), ref);
+			ref->priv->ref_object = obj;
+			
+			g_free (ref->priv->obj_name);
+			ref->priv->obj_name = g_strdup (gda_object_get_name (obj));
+			
 #ifdef GDA_DEBUG_signal
-		g_print (">> 'REF_FOUND' from %s\n", __FUNCTION__);
+			g_print (">> 'REF_FOUND' from %s\n", __FUNCTION__);
 #endif
-		g_signal_emit (G_OBJECT (ref), gda_object_ref_signals[REF_FOUND], 0);
+			g_signal_emit (G_OBJECT (ref), gda_object_ref_signals[REF_FOUND], 0);
 #ifdef GDA_DEBUG_signal
-		g_print ("<< 'REF_FOUND' from %s\n", __FUNCTION__);
+			g_print ("<< 'REF_FOUND' from %s\n", __FUNCTION__);
 #endif
+		}
+		else {
+			/* this is temporary, to satisfy the users of @ref which still need
+			 * to know what object is referenced when @ref is in fact losing its
+			 * referenced object */
+			ref->priv->ref_object = obj;
+		}
 	}
 
 	return ref->priv->ref_object ? TRUE : FALSE;
@@ -998,7 +1036,7 @@ gda_object_ref_activate (GdaObjectRef *ref)
  * gda_object_ref_deactivate
  * @ref: a #GdaObjectRef object
  *
- * Desctivates the object (loses the reference to the object)
+ * Desctivates the object (loses the reference to the referenced object)
  */
 void
 gda_object_ref_deactivate (GdaObjectRef *ref)
@@ -1018,7 +1056,8 @@ gda_object_ref_deactivate (GdaObjectRef *ref)
 		g_object_unref (ref->priv->ref_object);
 
 	ref->priv->ref_object = NULL;
-
+	ref->priv->ref_losing = TRUE;
+	g_object_ref (ref);
 	if (! ref->priv->block_signals) {
 #ifdef GDA_DEBUG_signal
 		g_print (">> 'REF_LOST' from %s\n", __FUNCTION__);
@@ -1028,6 +1067,9 @@ gda_object_ref_deactivate (GdaObjectRef *ref)
 		g_print ("<< 'REF_LOST' from %s\n", __FUNCTION__);
 #endif
 	}
+	ref->priv->ref_object = NULL;
+	ref->priv->ref_losing = FALSE;
+	g_object_unref (ref);
 }
 
 /**
